#
# Copyright (C) 2002 Jp Calderone.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of version 2.1 of the GNU Lesser General Public
# License as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import interfaces, error

from twisted.python import components

class SchedulerAdapter(components.Adapter):
    __implements__ = (interfaces.IScheduler,)

    def schedule(self, period, fn, *args, **kw):
        from twisted.internet import reactor
        def cycle():
            fn(*args, **kw)
            reactor.callLater(period, cycle)
        reactor.callLater(period, cycle)


class StorageAdapter(components.Adapter):
    __implements__ = (interfaces.IStorage,)

    def __init__(self, original):
        components.Adapter.__init__(self, original)
        self.storage = {}

    def store(self, key, version, value):
        self.storage.setdefault(key, {})[version] = value

    def retrieve(self, key, version):
        if key in self.storage:
            if version in self.storage[key]:
                return self.storage[key][version]
            raise error.StorageVersionError, (key, version)
        raise error.StorageKeyError, (key, version)


class ServicesAuthenticator(components.Adapter):
    __implements__ = (interfaces.IAuthenticator,)
    
    def __init__(self, irc):
        self.irc = irc

    def authenticate(self, name):
        return self.irc.whois(name).addCallback(self._checkIdentification)

    def _checkIdentification(self, lines):
        for (cmd, prefix, params) in lines:
            if cmd == '320':
                return 1
        return 0


class PasswordAuthenticator(components.Adapter):
    __implements__ = (interfaces.IAuthenticator,)
    
    def __init__(self, irc):
        self.irc = irc
        self.inProgress = {}
    
    def authenticate(self, name):
        if name not in self.irc.network.passwords:
            return defer.succeed(0)
        self.irc.msg(name, "Please provide your password for authentication (command 'password')")
        
        d = defer.Deferred()
        self.inProgress.setdefault(name, []).append(d)
        return d
    
    
    def checkResponse(self, user, response):
        if self.inProgress.has_key(user) and user in self.irc.network.passwords:
            if response == self.irc.network.passwords[user]:
                for d in self.inProgress[user]:
                    try:
                        d.callback(1)
                    except:
                        log.deferr()
            else:
                self.irc.msg(user, 'Authentication failed!')
            del self.inProgress[user]
        else:
            self.irc.msg(user, 'There is currently no authentication sequence in progress.')
