
from twisted.python import components

class IAuthenticator(components.Interface):
    def authenticate(self, user):
        """Attempt to authenticate the specified user.

        Return a Deferred the callback of which will be invoked with a
        true value of the user is authenticated, a false value otherwise.
        """

class IScheduler(components.Interface):
    # Convenience
    MINUTELY = 60
    HOURLY = 60 * 60
    DAILY = 60 * 60 * 24
    WEEKLY = 60 * 60 * 24 * 7


    def schedule(self, period, fn, *args, **kw):
        """
        Cause a function to be invoked at regular intervals with the given
        arguments.
        """

class IStorage(components.Interface):

    def store(self, key, version, value):
        """
        Store any pickleable object
        """

    def retrieve(self, key, version):
        """
        Retrieve the previously stored object associated with key and
        version
        """
