#
#   Twisted Pynfo - A relay, info, and search bot for IRC.
#   Copyright (C) 2002 Jp Calderone (exarkun@twistedmatrix.com)
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
#

import time

class Factoid:
    factoids = {}
    def __init__(self, key, body, creator=None, when=None):
        if when is None:
            when = time.time()
        if creator is None:
            creator = 'anonymous'
        key = key.strip().lower()
        self.key, self.body, self.creator, self.when = key, body, creator, when
        self.factoids[self.key] = self

    def lookup(klass, key):
        return klass.factoids.get(key.strip().lower())
    lookup = classmethod(lookup)

def filter_FACTOID(bot, user, channel, line):
    if line.startswith(bot.nickname):
        line = line.split(None, 1)
        if len(line) == 2:
            f = Factoid.lookup(line[1])
            if f:
                bot.reply(channel, f.body)


def info_TELL(bot, user, channel, args):
    """Usage: tell <user> about <item> - Inform the specified user about the specified item."""
    args = args.split(' about ', 1)
    if len(args) != 2:
        bot.reply(channel, "Could you be a little more clear?")
    else:
        who, what = args
        information = bot.factory.lookupFact(what)
        if not information:
            bot.reply(channel, "But I don't know a thing about that.");
        else:
            if who == 'us':
                who = channel
            elif who == 'me':
                who = user
            else:
                bot.reply(channel, 'Alrighty.')
            if who != user:
                bot.reply(user, "I am telling %s: %s" % (who, information))
            bot.reply(who, "%s asked me to tell you this: %s" % (user, information))


def info_DEFINE(bot, user, channel, args):
    """Usage: define <factoid> as <definition> - Create or redefine a factoid."""
    args = args.split(' as ', 1)
    if len(args) != 2:
        bot.reply(channel, "Incorrect syntax.  See 'help define'")
    else:
        bot.factory.setFact(*args)
        bot.reply(channel, 'Defined.')


def privinfo_FORGET(bot, user, channel, args):
    """Usage: forget <factoid> - Remove a factoid."""
    try:
        bot.factory.unsetFact(args)
    except KeyError:
        bot.reply(channel, "No such factoid.")
    else:
        bot.reply(channel, "Forgotten.")
