#
#   Twisted Pynfo - A relay, info, and search bot for IRC.
#   Copyright (C) 2002 Jp Calderone (exarkun@twistedmatrix.com)
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
#

"""
Commands for controlling Pynfo's behavior on an IRC network.
"""

import random

def privinfo_JOIN(bot, user, channel, args):
    """Usage: join <channel>[@network] - Join the specified channel"""
    try:
        newchannel, network = args.split('@', 1)
    except ValueError, e:
        newchannel, network = args, bot.network.alias
    try:
        bot = bot.factory.allBots[network]
    except KeyError:
        bot.reply(channel, "I am not on that network.")
    else:
        if newchannel in bot.network.channels:
            bot.reply(channel, "I am already in %s on %s." % (newchannel, network))
        else:
            bot.join(newchannel)


def privinfo_PART(bot, user, channel, args):
    """Usage: part <channel>[@network] - Leave the specified channel"""
    try:
        newchannel, network = args.split('@', 1)
    except ValueError, e:
        newchannel, network = args, bot.network.alias
    try:
        bot = bot.factory.allBots[network]
    except KeyError:
        bot.reply(channel, "I am not on that network.")
    else:
        if newchannel not in bot.network.channels:
            bot.reply(channel, "I am not in %s on %s." % (newchannel, network))
        else:
            bot.network.channels.remove(newchannel)
            bot.part(newchannel)


def privinfo_LOGOFF(bot, user, channel, args):
    """Usage: logoff - Leave this network"""
    bot.quit(random.choice(bot.QUIT_MSGS))
    bot.hasQuit = 1


def privinfo_RECONNECT(bot, user, channel, args):
    """Usage: reconnect [network] - Disconnect and reconnect to the given network, or this network if none is specified"""
    if args:
        if bot.factory.allBots.has_key(args):
            bot.factory.allBots[args].quit(random.choice(bot.QUIT_MSGS))
        else:
            bot.reply(channel, "No such network")
    else:
        bot.quit(random.choice(bot.QUIT_MSGS))


def privinfo_CONNECT(bot, user, channel, args):
    """Usage: connect <alias> <address>[:port] - Connect to the given network"""
    try:
        alias, host, port = bot.parseNetwork(args)
    except ValueError, e:
        bot.reply(channel, "Incorrect usage.  See 'help connect'")
        return

    try:
        n = bot.factory.networkByAlias(alias)
        bot.reply(channel, "%s is already an alias to %s." % (alias, n.address))
        return
    except KeyError:
        pass

    bot.factory.createNetwork((host, port), alias, bot.nickname)
    
    from twisted.internet.app import theApplication
    theApplication.connectTCP(host, port, bot.factory)
    bot.reply(channel, "Connection initiated to %s:%d (alias %s)" % (host, port, alias))


def privinfo_DISCONNECT(bot, user, channel, args):
    """Usage: disconnect <alias> - Disconnect from the network with the specified alias"""
    try:
        n = bot.factory.networkByAlias(args)
    except KeyError:
        bot.reply(channel, "I am not connected to %s." % (args,))
    else:
        if len(bot.factory.data['networks']) == 1:
            bot.reply(channel, "But %s is the last network to which I am connected!" % (args,))
        else:
            # XXX - This is so bad.
            try: bot.factory.allBots[n.alias].quit()
            except KeyError: pass
            try: del bot.factory.data['networks'][n.alias]
            except KeyError: pass
            try: del bot.factory.allBots[n.alias]
            except KeyError: pass

            from twisted.internet.app import theApplication as app
            app.tcpConnectors = [c for c in app.tcpConnectors if c[:2] != n.address]

            bot.reply(channel, "Disconnected from %s." % (args,))
