# -*- coding: Latin-1 -*-

import re

from Pynfo import info
from Pynfo import error

from cvstoys import common
from cvstoys.clients import words
from cvstoys.clients.pb import PerspectiveInUse

from twisted.spread import pb
from twisted.spread import jelly
from twisted.python import log
from twisted.internet import reactor
from twisted.internet import defer
from twisted.python import components

# rape/paste to allow customizable security
def getObjectRetriever():
    d = defer.Deferred()
    security = jelly.SecurityOptions()
    security.allowBasicTypes()
    security.allowTypes('CVSToys.VersionedPatch')
    b = pb.Broker(1)
    bf = pb.BrokerClientFactory(b)
    pb._ObjectRetrieval(b, d)
    return bf, d

def getObjectAt(host, port, timeout=None):
    bf, d = getObjectRetriever()  
    reactor.connectTCP(host, port, bf, timeout)
    return d

def connect(host, port, username, password, serviceName,
            perspectiveName=None, client=None, timeout=None):  
    d = defer.Deferred()
    getObjectAt(host,port,timeout).addCallbacks(
        pb._connGotRoot, d.errback,
        callbackArgs=[d, client, serviceName,
                      username, password, perspectiveName])
    return d

class FailedConnection(Exception):
    pass

class Notifier(pb.Referenceable):
    def __init__(self, factory, host, port, username, password, serviceName):
        self.factory = factory
        self.host = host
        self.port = int(port)
        self.username = username
        self.password = password
        self.serviceName = serviceName
        self.notify = {}
    
    def __str__(self):
        return '<Notifier: %s@%s:%d, %s>' % (self.username, self.host, self.port, self.serviceName)

    def connect(self, reportErrors):
        d = connect(self.host, self.port, self.username, self.password, self.serviceName, client=self)
        d.addCallbacks(self.connected, self.failed, errbackArgs=(reportErrors,))
        return d

    def disconnect(self):
        self.remote_notify = lambda *args: None
        self.remote_goodbye = lambda *args: None
        # XXX - Actually disconnect

    def connected(self, perspective):
        perspective.notifyOnDisconnect(self.disconnected)
        for (net, chans) in self.notify.items():
            bot = self.factory.allBots.get(net)
            if bot:
                for chan in chans:
                    bot.msg(chan, 'CVS notifier connected')

    def disconnected(self, _):
        if not self.factory.allBots:
            return
        for (net, chans) in self.notify.items():
            bot = self.factory.allBots.get(net)
            if bot:
                for chan in chans:
                    bot.msg(chan, 'CVS notifier disconnected')
    
    def failed(self, failure, chan):
        if failure.check(PerspectiveInUse):
            for bot in self.factory.allBots.values():
                if chan in bot.network.channels:
                    bot.msg(chan, "Perspective already in use!  I give up.")
            raise FailedConnection()
        else:
            log.err(failure)
            reactor.callLater(120, self.connect)

    def remote_notify(self, root, files, message, user):
        s = words.makeMessage(root, files, message, user)
        for (net, chans) in self.notify.items():
            bot = self.factory.allBots.get(net)
            if bot:
                for (chan, match) in chans.items():
                    for (expr, cexpr) in match:
                        if filter(cexpr.search, [f.pathname for f in files]):
                            print expr, 'matched something of', [f.pathname for f in files]
                            bot.msg(chan, s)
                            break

    def remote_goodbye(self, message):
        for (net, chans) in self.notify.items():
            bot = self.factory.allBots.get(net)
            if bot:
                for chan in chans:
                    bot.msg(chanl, 'Server said ' + message)

def delkey(failure, dict, key):
    failure.trap(FailedConnection)
    del dict[key]

def privinfo_NOTIFIER(bot, user, channel, args):
    """Usage: notifier <name> [<host> <port> <username> <password> <servicename>] - Add or remove a notifier server"""
    d = INotifierManager(bot.factory).notifiers
    parts = args.split()
    if len(parts) == 1:
        if parts[0] in d:
            d[parts[0]].disconnect()
            del d[parts[0]]
            bot.msg(channel, "Disconnecting...")
        else:
            bot.msg(channel, "No such notifier")
    elif len(parts) == 6:
        if parts[0] in d:
            bot.msg(channel, "Notifier by that name already exists")
        else:
            d[parts[0]] = Notifier(bot.factory, *parts[1:])
            d[parts[0]].connect(channel).addErrback(delkey, d, parts[0])
            bot.msg(channel, "Connecting...")
    else:
        raise error.UsageError

def privinfo_WATCH(bot, user, channel, args):
    """Usage: watch <notifier name> <regular expression> - Add a watch to the given notifier for this channel"""
    d = INotifierManager(bot.factory).notifiers
    parts = args.split(None, 1)
    if len(parts) == 2:
        name, expr = parts
        if name not in d:
            bot.msg(channel, "No such notifier.")
            return
        n = d[name]
        try:
            cexpr = re.compile(expr)
        except:
            bot.msg(channel, "Illegal expression")
            return
        n.notify.setdefault(bot.network.alias, {}).setdefault(channel, []).append((expr, cexpr))
        bot.msg(channel, "Okay.")
    else:
        raise error.UsageError

def info_LISTWATCH(bot, user, channel, args):
    """Usage: listwatch <notifier name> - List watches for the given notifier for this channel"""
    d = INotifierManager(bot.factory).notifiers
    if args in d:
        if bot.network.alias in d[args].notify and channel in d[args].notify[bot.network.alias]:
            for (expr, _) in d[args].notify[bot.network.alias][channel]:
                bot.msg(channel, expr)
        else:
            bot.msg(channel, "No watches for that notifier for this channel.")
    else:
        bot.msg(channel, "No such notifier.")

def info_LISTNOTIFIERS(bot, user, channel, args):
    """Usage: listnotifiers - List all current notifiers"""
    d = INotifierManager(bot.factory).notifiers
    if d:
        for (k, v) in d.items():
            bot.msg(channel, '%s: %s' % (k, v))
    else:
        bot.msg(channel, 'No notifiers configured.')

class INotifierManager(components.Interface):
    """
    @ivar notifiers: Dictionary mapping notifier names to notifier references.
    """

class NotifierAdapter(components.Adapter):
    __implements__ = (INotifierManager,)
    def __init__(self, original):
        components.Adapter.__init__(self, original)
        self.notifiers = {}

components.registerAdapter(NotifierAdapter, info.InfoBotFactory, INotifierManager)
