#
#   Twisted Pynfo - A relay, info, and search bot for IRC.
#   Copyright (C) 2002 Jp Calderone (exarkun@twistedmatrix.com)
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
#

import time, types, bisect, string, urllib

from twisted.python import log

def _lower(key):
    if isinstance(key, types.StringType):
        return key.lower()
    return key


class CaseDict:
    def __init__(self, d=None):
        if d is None:
            d = {}
        self.d = dict([(k, (k, v)) for (k, v) in d.items()])


    def has_key(self, key):
        return self.d.has_key(_lower(key))
    __contains__ = has_key


    def __getitem__(self, key):
        return self.d[_lower(key)][1]


    def get(self, key, default=None):
        return self.d.get(_lower(key), default)


    def __setitem__(self, key, value):
        self.d[_lower(key)] = (key, value)


    def __delitem__(self, key):
        del self.d[_lower(key)]


    def orig_key(self, key):
        return self.d[_lower(key)][0]


    def keys(self):
        return self.d.keys()


    def values(self):
        return [x[1] for x in self.d.values()]


    def items(self):
        return self.d.values()


    def __len__(self):
        return len(self.d)


    def __repr__(self):
        return 'CaseDict(%r)' % self.d


    def __iter__(self):
        return iter(self.d)


class CaseList:
    origList = None
    lowerList = None
    
    def __init__(self, sequence = ()):
        self.origList = []
        self.lowerList = []
        for i in sequence:
            self.append(i)
    
    
    def __contains__(self, element):
        return self.lowerList.__contains__(_lower(element))
    
    
    def __getitem__(self, index):
        return self.origList[index]
    
    
    def __setitem__(self, index, value):
        self.origList[index] = value
        self.lowerList[index] = _lower(value)


    def __delitem__(self, index):
        del self.origList[index]
        del self.lowerList[index]
    

    def __eq__(self, other):
        if isinstance(other, CaseList):
            return self.lowerList.__eq__(other.lowerList)
        else:
            return self.lowerList.__eq__(other)
    
    
    def __lt__(self, other):
        if isinstance(other, CaseList):
            return self.lowerList.__lt__(other.lowerList)
        else:
            return self.lowerList.__eq__(other)
    
    
    def __le__(self, other):
        if isinstance(other, CaseList):
            return self.lowerList.__le__(other.lowerList)
        else:
            return self.lowerList.__eq__(other)


    def __len__(self):
        return len(self.origList)
    
    
    def append(self, value):
        self.origList.append(value)
        self.lowerList.append(_lower(value))


    def remove(self, value):
        i = self.lowerList.index(_lower(value))
        del self.origList[i], self.lowerList[i]


    def index(self, value):
        return self.lowerList.index(_lower(value))


def findConversation(conv, logs):
    end = None
    s = conv.started
    e = conv.ended or time.time()

    begin = bisect.bisect(logs, [s])
    if conv.ended is not None:
        end = bisect.bisect(logs, [e])
        r = logs[begin:end]
    else:
        r = logs[begin:]

    return r

def safeFilename(filename, safe=string.letters + string.digits):
    return urllib.quote(filename, safe=safe)
